package eu.gronos.kostenrechner.controller.system;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractButton;

/**
 * Ein {@link ItemListener} für {@link AbstractButton}-Elemente.
 * 
 * Jede Instanz korrespondiert mit einem Boolean-Wert <code>schluessel</code> in
 * der Registry im Unterast <code>"/eu/gronos/kostentenor"</code> zum Java-Ast.
 * Diese wird zunächst eingelesen. Wenn der Schlüssel <code>schluessel</code>
 * noch nicht gesetzt ist, wird der als <code>standardWert</code> übergebene
 * Wahrheitswert genommen.
 *
 * @author Peter Schuster (setrok)
 * @date 18 Feb 2019
 *
 */
public class BooleanConfigOptionHorcher extends BooleanConfigOption implements ItemListener {

	private AbstractButton button;

	/**
	 * @param schluessel   der Schlüssel im Unterast
	 *                     <code>"/eu/gronos/kostentenor"</code> (String)
	 * @param standardWert der Standard-Wert, falls der Schlüssel noch nicht
	 *                     angelegt ist. (boolean)
	 * @param button       der {@link AbstractButton}, an dem die Option an der
	 *                     Oberfläche geändert werden kann und für den die Instanz
	 *                     als {@link ItemListener} eingerichtet werden soll.
	 */
	public BooleanConfigOptionHorcher(String schluessel, boolean standardWert, AbstractButton button) {
		super(schluessel, standardWert);
		setButton(button);
		getValue();
	}

	/**
	 * Die Methode schreibt den Wert in die Registry/Preferences und setzt das
	 * Oberflächenelement entsprechend
	 * 
	 * @param value
	 */
	public void setValue(boolean value) {
		super.setValue(value);
		if (getButton() != null) {
			getButton().setSelected(value);
		}
	}

	/**
	 * 
	 * Die Methode liest den Wert aus der Registry (den Preferences) und setzt das
	 * Oberflächenelement entsprechend
	 * 
	 * @return value als boolean
	 */
	public boolean getValue() {
		boolean value = super.getValue();
		if (getButton() != null) {
			getButton().setSelected(value);
		}
		return value;
	}

	/**
	 * Die Methode wird aufgerufen, wenn sich der Status
	 * {@link AbstractButton#isSelected()} ändert.
	 * 
	 * @param e ein {@link ItemEvent}
	 * 
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
		boolean value = getButton().isSelected();
		inRegistrySchreiben(value);
	}

	/**
	 * Die Methode setzt wieder den Standardwert in die Registry/Preferences.
	 * 
	 * @see eu.gronos.kostenrechner.controller.system.BooleanConfigOption#reset()
	 */
	@Override
	public void reset() {
		setValue(getStandardWert());
	}

	/**
	 * @return gibt den {@link AbstractButton}, an dem die Option an der Oberfläche
	 *         geändert werden kann und für den die Instanz als {@link ItemListener}
	 *         eingerichtet werden soll.
	 */
	public AbstractButton getButton() {
		return button;
	}

	/**
	 * @param button der {@link AbstractButton}, an dem die Option an der Oberfläche
	 *               geändert werden kann und für den die Instanz als
	 *               {@link ItemListener} eingerichtet werden soll.
	 */
	public void setButton(AbstractButton button) {
		this.button = button;
	}

}
