/*
 * FontHelfer.java
 * eu.gronos.kostenrechner.view.result (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JTextPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;

import eu.gronos.kostenrechner.Kostenrechner;

/**
 * Die Klasse dient dazu, Fonts zu suchen, Formatierungen zu setzen und die
 * Fenstergröße nach Schriftgröße zu setzen.
 *
 * @author Peter Schuster (setrok)
 * @date 19 Jun 2019
 *
 */
public class FontHelfer {

	/**
	 * maximale Anzahl der Zeilen für die Größe des Dialogfensters
	 */
	public static final int MAX_ZEILEN = 40;
	/**
	 * maximale Anzal der Spalten für die Größe des Dialogfensters
	 */
	public static final int MAX_SPALTEN = 150;

	/**
	 * Konstruktor.
	 * 
	 */
	public FontHelfer() {
		super();
	}

	/**
	 * Die Methode dient dazu, die Breite eines Zeichens ('n') und die
	 * Standardzeilenhöhe zu ermitteln
	 * 
	 * @param fontMetrics
	 * @return Zeichenbreite und Zeilenhöhe in Pixeln als {@link Dimension}, also
	 *         eine {@link Dimension} aus {@link FontMetrics#charWidth(int)} und
	 *         {@link FontMetrics#getHeight()}
	 */
	public Dimension getCharDimension(FontMetrics fontMetrics) {
		int charWidth = fontMetrics.charWidth('n');// 'm'
		int lineHeight = fontMetrics.getHeight();
		Kostenrechner.getLogger()
				.info("Breite eines 'n':" + charWidth + "; Höhe des Fonts: " + fontMetrics.getHeight());
		return new Dimension(charWidth, lineHeight);
	}

	/**
	 * Die Methode dient dazu, die Schriftart Arial , sonst eine verwandte
	 * Schriftart aus den Schriftarten des Systems zu finden. Sie geht nacheinander
	 * die Schriftnamen durch. Ist einer davon der aktuelle Font, gewinnt dieser
	 * immer, sonst wird einer der anderen aus der Liste genommen, sofern er
	 * existiert.
	 * 
	 * @param attrib ein {@link MutableAttributeSet} oder {@link Style}
	 */
	public boolean setFontArial(MutableAttributeSet attrib) {
		boolean ok = false;
		final String ff = StyleConstants.getFontFamily(attrib);
		for (String arial : new String[] { "Arial", "Helvetica", "Helv" })
			if (arial.equals(ff)) {
				StyleConstants.setFontFamily(attrib, arial);
				ok = true;
			} else if (!ok && getFontForName(arial) != null) {
				StyleConstants.setFontFamily(attrib, arial);
				ok = true;
			}
		return ok;
	}

	/**
	 * Die Methode definiert erstmal den grundlegenden Style ("base"), wobei diese
	 * direkt ins übergebene {@link StyledDocument} geschrieben werden. Zusätzlich
	 * wird der Style noch zurückgegeben.
	 * 
	 * @param doc  ein {@link StyledDocument}
	 * @param font ein {@link Font}
	 * @return ein {@link Style} mit den grundlegenden Formatierungen
	 * @see #createBaseStyle(StyledDocument)
	 */
	public Style createBaseStyle(StyledDocument doc, final Font font) {
		Style baseStyle = doc.addStyle("base", null);
		setFontArial(baseStyle);
		setFontSize(baseStyle, font);
	
		StyleConstants.setLineSpacing(baseStyle, .33f);
		StyleConstants.setSpaceBelow(baseStyle, 7f);
	
		return baseStyle;
	}

	/**
	 * Die Methode baut einen weiteren Style für fetten Text und schreibt ihn ins
	 * übergebene {@link StyledDocument}. Zusätzlich wird der Style noch
	 * zurückgegeben.
	 * 
	 * @param doc       ein {@link StyledDocument}
	 * @param baseStyle ein {@link Style} mit den grundlegenden Formatierungen
	 * @return ein {@link Style} mit den neuen Formatierungen
	 * @see #createBaseStyle(StyledDocument)
	 */
	public Style createBoldStyle(StyledDocument doc, Style baseStyle) {
	
		Style style = doc.addStyle("bold", baseStyle);
		StyleConstants.setBold(style, true);
	
		return style;
	}

	/**
	 * Die Methode baut einen Style mit Tabstopps ("tabulatoren") und schreibt ihn
	 * ins übergebene {@link StyledDocument}. Zusätzlich wird der Style noch
	 * zurückgegeben.
	 * 
	 * @param doc       ein {@link StyledDocument}
	 * @param baseStyle ein {@link Style} mit den Formatierungen
	 * @return ein {@link Style} mit den neuen Formatierungen
	 * @url "http://www.java2s.com/Tutorial/Java/0240__Swing/TabStopandTabSetClasses.htm"
	 *      "http://www.java2s.com/Code/Java/Swing-JFC/Createadecimalalignedtabstopat400pixelsfromtheleftmargin.htm"
	 * @see #createBaseStyle(StyledDocument)
	 */
	public Style createTabStoppedStyle(StyledDocument doc, Style baseStyle) {
		Style style = doc.addStyle("tabulatoren", baseStyle);
		StyleConstants.setTabSet(style,
				new TabSet(new TabStop[] { new TabStop(600, TabStop.ALIGN_RIGHT, TabStop.LEAD_DOTS),
						new TabStop(950, TabStop.ALIGN_RIGHT, TabStop.LEAD_DOTS) }));
	
		return style;
	}

	/**
	 * Die Methode setzt die Absatzformatierung auf anderthalbfachen
	 * Zeilenabsabstand (ok, 1,33-fach) und 7px Abstand danach.
	 * 
	 * @param textPane          ein {@link JTextPane}
	 * @param doc               ein {@link StyledDocument}
	 * @param offset            the start of the change >= 0
	 * @param length            the length of the change >= 0
	 * @param grosseTabulatoren sollen große Tabulatorabstände eingebaut werden?
	 * @see javax.swing.JTextPane#getParagraphAttributes()
	 */
	public void setParagraphAttributes(JTextPane textPane, StyledDocument doc, int offset, int length,
			boolean grosseTabulatoren) {
		int caret = textPane.getCaretPosition();
		textPane.setCaretPosition(offset);
		if (grosseTabulatoren)
			doc.setParagraphAttributes(offset, length, doc.getStyle("tabulatoren"), false);
		else
			doc.setParagraphAttributes(offset, length, doc.getStyle("base"), false);
		textPane.setCaretPosition(caret);
	}

	/**
	 * Die Methode findet die Schriftgröße heraus. Die Schriftgröße wird auf 12px
	 * gesetzt, solange nicht die Systemschriftgröße größer ist.
	 * 
	 * @param attrib ein {@link MutableAttributeSet} oder {@link Style}
	 * @param font   der {@link Font}
	 */
	public void setFontSize(MutableAttributeSet attrib, final Font font) {
		// TODO Kann man herausfinden, ob die Schriftart die Größe unterstützt?
		final int fs = StyleConstants.getFontSize(attrib);
		StyleConstants.setFontSize(attrib, fs > font.getSize() ? fs : font.getSize());
	}

	/**
	 * Die Methode errechnet für eine {@link JComponent} die Maße, dass sie nicht
	 * breiter als maxSpalten Zeichen und nicht höher als maxZeilen Zeilen wird.
	 * 
	 * @param attrib        ein {@link AttributeSet}
	 * @param maxSpalten    Spaltenzahl für die Berechnung
	 * @param maxZeilen     Zeilenanzahl für die Berechnung
	 * @param charDimension Breite und Höhe eines Buchstaben als {@link Dimension},
	 *                      {@link #getCharDimension(FontMetrics)}
	 * @param maximumSize   {@link JComponent#getMaximumSize()}
	 * @param preferredSize {@link JComponent#getPreferredSize()}
	 */
	public Dimension calculatePreferredSizeFrom(AttributeSet attrib, int maxSpalten, int maxZeilen,
			Dimension charDimension, Dimension maximumSize, Dimension preferredSize) {
		// Dimension charDimension = getCharDimension(attrib);
		/* Breite */
		if (charDimension.width > 0 && (charDimension.width * maxSpalten) < maximumSize.width) {
			preferredSize.width = charDimension.width * maxSpalten;
		} else {
			preferredSize.width = maximumSize.width;
		}
		/* Höhe */
		if (charDimension.height > 0 && (charDimension.height * maxZeilen) < maximumSize.height) {
			preferredSize.height = charDimension.height * maxZeilen;
		}
		Kostenrechner.getLogger()
				.info("maximumSize:" + maximumSize.toString() + "; preferredSize: " + preferredSize.toString());
		return preferredSize;
	}

	/**
	 * Die Methode ermittelt die bevorzugte Größe des {@link JDialog}s, indem sie
	 * die Größe der Ränder hinzurechnet.
	 * 
	 * @param insets    {@link Insets} des {@link JDialog}s
	 * @param minimum   {@link JComponent#getMinimumSize()}
	 * @param preferred {@link JComponent#getPreferredSize()}
	 * @return eine {@link java.awt.Dimension} mit der vergrößerten bevorzugten
	 *         Größe
	 */
	public Dimension calculatePreferredSizeWith(Insets insets, final Dimension minimum, final Dimension preferred) {
		// Insets insets = border.getBorderInsets();
		double rand = (double) (insets.left + insets.right + insets.top + insets.bottom);
		// Liefert getMinimumSize() oder getPreferredSize() den größeren Wert
		// für Breite und Höhe?
		// Nimm die größere Breite
		double width = minimum.getWidth() > preferred.getWidth() ? minimum.getWidth() : preferred.getWidth();
		// Nimm die größere Höhe
		double height = minimum.getHeight() > preferred.getHeight() ? minimum.getHeight() : preferred.getHeight();
		// Rand hinzurechnen
		width += rand;
		height += rand;
		return new Dimension((int) width, (int) height);
	}

	/**
	 * Die Methode getFontForName dient dazu, herauszufinden, ob ein Font mit dem
	 * als <code>name</code> übergebenen Namen im System existiert.
	 * 
	 * @param name Suchbegriff Name des Fonts
	 * @return den Font des übergebenen Namens, falls gefunden.
	 */
	protected Font getFontForName(String name) {
		if (name == null || "".equals(name))
			return null;
		Font f = null;
		for (String fonts : GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames())
			if (name.equals(fonts))
				f = new Font(fonts, Font.PLAIN, 12);
		return f;
	}
}